; SID+Midi player
; DASM source
; 4mat/Ate Bit - 1/4/2019

; Midi driver is at $c000-$cfff along with it's variables/buffers. Zeropage $02+$03 are used for temp data.

; $c000 - Init:
; Set the X,Y,A registers to change these parameters:
; X - SID Buffer value (0-7)
; Y - Cart type - 0 - Sequential , 1 - Datel
; A - How many SID songs are being used. (1-4) For example, if you're using just one song set this to 1.
  
; $c003 - Play (each frame)

; $c006 - Stop (resets midi device and turns off sid)

	org $c000

	; If set to 1 uses an NTSC table for the pitch detection.
	NTSC = 0
	
	jmp mididriver_init
	jmp mididriver_play
	jmp mididriver_stop
	
mididriver_stop

	jsr resetmidi
	rts
	
mididriver_init

	sta player_patchmax
	
	stx buffernumber

	tya
	asl
	asl
	asl
	tay
	lda midicartdata,y
	sta resetcontrol+$01
	sta resetcontrol2+$01
	iny
	lda midicartdata,y
	sta resetirq+$01
	iny
	lda midicartdata,y
	sta writedatawrite+$01
	iny
	lda midicartdata,y
	sta writetransmit+$01
	iny
	lda midicartdata,y
	sta resetmidival+$01
	
	jsr resetmidi

	jsr patch_sids

	ldx #$00
initdrivers
	stx drivernumber
	lda player_playhi,x
	sta initit+$02
	lda #$00
initit
	jsr $4800
	ldx drivernumber
	inx
	cpx player_patchmax
	bne initdrivers

	ldx #$1f
resetmidikeyset
	lda #$ff
	sta midichancurrentchannel,x
	sta notedelay,x
	lda #$00
	sta midichancurrentnote,x
	dex
	bpl resetmidikeyset
	
	ldx #$0f
resetmidikeypatch	
	lda #$ff
	sta midirealpatch,x
	lda #$00
	sta gateset,x
	dex
	bpl resetmidikeypatch
	
	ldx #$03
setplaydelay	
	lda player_startdelay,x
	sta tempplayer_startdelay,x
	dex
	bpl setplaydelay
	
	lda #$00
	sta buffercurrent
	
timerreset	
	ldx #$00
	stx temp
timersync	
	stx drivernumber
	lda tempplayer_startdelay,x
	bpl timersynccont
	inc temp
	jmp timersyncskip
timersynccont
	lda player_playhi,x
	sta timersyncplay+$02
	lda player_playlo,x
	sta timersyncplay+$01
	dec tempplayer_startdelay,x
timersyncplay	
	jsr $1003 
timersyncskip
	ldx drivernumber
	inx
	cpx player_patchmax
	bne timersync
	lda temp
	cmp player_patchmax
	bne timerreset	
	
mididriver_initend	
	rts
	
		
mididriver_play

	; play sid driver(s)
	
	lda #$00
	sta midisendcount

	ldx #$00
	
songloop
	stx drivernumber
	
	lda player_playhi,x
	sta playit+$02
	lda player_playlo,x
	sta playit+$01
	
playit	
	jsr $1003 
	

	
	ldx drivernumber
	inx
	cpx player_patchmax 
	beq songendloop
	jmp songloop
	
songendloop

	
	
	; interpret midi 
	
	ldy #$00
chanupdates
	
	sty midichantemp


	

	lda midichancurrentchannel,y
	bpl normalmidi
	jmp instrumentcheck
	
normalmidi

	; check delay before sending note off
	lda notedelay,y
	bmi nonotedelay
	clc
	lda notedelay,y
	sbc #$01
	sta notedelay,y
	bpl nonotedelay
	
	; send note off here

	ldx midisendcount
	clc
	lda midichancurrentchannel,y
	adc #$80
	sta midibuffer,x
	lda midichancurrentnote,y
	sta midibuffer+$01,x
	lda midichancurrentvelocity,y
	sta midibuffer+$02,x
	inx
	inx
	inx
	stx midisendcount

	
	
nonotedelay
	
	; check for modulation
	
	lda modplaying,y
	cmp #$01
	beq nomoddelay
	
	lda moddelay,y
	cmp #$ff
	beq nomoddelay
	
	clc
	lda moddelay,y
	sbc #$00
	sta moddelay,y
	
	cmp #$ff
	bne nomoddelay


	ldx midisendcount	
	clc
	lda midichancurrentchannel,y
	adc #$b0
	sta midibuffer,x
	lda #$01
	sta modplaying,y
	sta midibuffer+$01,x
	lda midimodulationval,y
	sta midibuffer+$02,x
	
	inx
	inx
	inx
	stx midisendcount

	
nomoddelay
	
	; if gate has been set, we only want to look for a gate off in the waveform otherwise we ignore everything else
	
	lda gateset,y
	cmp #$01
	bne gateisntset
	
	ldx chanoffsets,y
	lda siddata+$04,x
	and #$01
	cmp #$00
	beq gateisset
	jmp loop2
gateisset
	sta gateset,y

	lda midifixeddelay,y
	cmp #$ff
	bne usefixeddelay	
	lda siddata+$06,x
	and #$0f
	asl
	asl

usefixeddelay	
	sta notedelay,y
	jmp loop2

gateisntset

	; if gate isn't set, check song for a gate which means we'd have a new note.
	
	ldx chanoffsets,y
	lda siddata+$04,x
	and #$01
	cmp #$01
	beq newnotestartprecheck
newnoteskip
	jmp loop2
newnotestartprecheck	
	lda siddata+$04,x
	and #$f0
	cmp #$00
	beq newnoteskip
	cmp #$90
	bcs newnoteskip

	; we have a gate and it's a new note to play

	; check if note delay is still active so we can turn off the current note

	jsr checknotedelay
		
	; check instrument


instrumentcheck

	jsr checkinstrument

		
	ldy midichantemp
	ldx chanoffsets,y
	lda midichancurrentchannel,y
	bpl actualmidi
	lda #$00
	sta gateset,y
	lda #$ff
	sta notedelay,y
	sta moddelay,y

	jmp loop2
	
actualmidi
	; find note, if it's zero we don't have a note to play after all
	
	lda siddata+01,x
	cmp #$00
	bne thereisnote
	lda #$00
	sta gateset,y
	jmp loop2
thereisnote	
	sta tempnotehi
	lda siddata,x
	sta tempnotelo	
	lda #$01
	sta gateset,y


	lda midihasmodulation,y
	cmp #$ff
	beq nomodreset
	lda modplaying,y
	cmp #$01
	bne domodreset

	ldx midisendcount
	clc
	lda midichancurrentchannel,y
	adc #$b0
	sta midibuffer,x
	lda #$01
	sta midibuffer+$01,x
	lda #$00 
	sta midibuffer+$02,x
	inx
	inx
	inx
	stx midisendcount

domodreset
	lda midihasmodulation,y
	sta moddelay,y
	lda #$00
	sta modplaying,y

	
	

nomodreset	
	ldx midisendcount
	clc
	lda #$90 
	adc midichancurrentchannel,y
	sta midibuffer,x
	
	lda midifixednotevalue,y
	bpl afixednote
	cmp #$00
	beq afixednote
		
	jsr findnoteval

	
	clc
	lda midival
	adc miditranspose,y
afixednote	
	sta midichancurrentnote,y
	ldx midisendcount
	sta midibuffer+$01,x
	
	lda midifixedvelocity,y
	bpl afixedvelocity
	ldx chanoffsets,y
	lda siddata+$06,x
	and #$f0
	lsr
	lsr
	lsr
	lsr
	tax
	lda volumetable,x 
	
afixedvelocity	
	sta midichancurrentvelocity,y
	ldx midisendcount
	sta midibuffer+$02,x
	inx
	inx
	inx
	stx midisendcount

loop2
	
	ldy midichantemp
	iny
	cpy #$09
	beq endchanupdates
	jmp chanupdates
	
endchanupdates
	
	lda midisendcount
	cmp #$00
	beq skipmidi
	
	jsr writemididata
	
skipmidi
	

	
	jsr writesid
	rts
	
	; generic write midi data stream
	
writemididata
	ldx #$00	
writedatawrite	
	lda $de02
	and #$02
	beq writedatawrite	
		
	lda midibuffer,x
writetransmit
	sta $de01 

	inx
	cpx midisendcount 
	bne writedatawrite
	rts
	
	
	
checkinstrument	


	ldx chanoffsets,y
	lda siddata+$04,x	
	and #$f0
	sta $50,y
	cmp #$00
	beq isnotinstrument
	cmp midichancurrentpatch,y
	bne isnewinstrument
isnotinstrument	
	jmp nonewinstrument
isnewinstrument
	sta midichancurrentpatch,y
	lda patchoffsetslo,y
	sta $f0
	lda patchoffsetshi,y
	sta $f1
	lda midichancurrentpatch,y
	lsr
	lsr
	lsr
	lsr
	tax
	lda hiloconv,x
	clc
	adc $f0
	sta $f0

	ldy #$00
	lda ($f0),y
	
	ldy midichantemp
	
	sta midichancurrentchannel,y
	bmi nonewinstrument
	
	tax
	ldy #$01
	lda ($f0),y
	cmp midirealpatch,x 
	beq skippatchchangehw
	; do instrument change on this midi channel	
	sta midirealpatch,x
	sty temp
	ldy midichantemp
	lda midichancurrentchannel,y
	
waitinsttrans1

	lda midichancurrentchannel,y
	clc
	adc #$c0
	pha
waitinsttrans2

	lda midirealpatch,x
	ldx midisendcount
	sta midibuffer+$01,x
	pla
	sta midibuffer,x
	inx
	inx
	stx midisendcount
	ldy temp
	


skippatchchangehw
	iny
	lda ($f0),y
	pha
	iny
	lda ($f0),y
	pha
	iny
	lda ($f0),y
	pha
	iny
	lda ($f0),y
	pha
	iny
	lda ($f0),y
	pha
	iny
	lda ($f0),y
	ldy midichantemp
	sta miditranspose,y
	pla
	sta midimodulationval,y
	pla
	sta midihasmodulation,y
	pla 
	sta midifixeddelay,y
	pla
	sta midifixedvelocity,y
	pla
	sta midifixednotevalue,y
	



			
	ldx chanoffsets,y	
nonewinstrument	
	rts

	
checknotedelay
	lda notedelay,y
	bmi noquellnote

	
	lda midichancurrentchannel,y
	bmi noquellnote
	
	lda #$ff
	sta notedelay,y
	
	ldx midisendcount
	clc
	lda midichancurrentchannel,y
	adc #$80
	sta midibuffer,x
	lda midichancurrentnote,y
	sta midibuffer+$01,x
	lda midichancurrentvelocity,y
	sta midibuffer+$02,x
	inx
	inx
	inx
	stx midisendcount
	

noquellnote
	rts

resetmidi
	ldx #$18
	lda #$00
clearoutsid
	sta $d400,x
	sta siddata,x
	sta sidbuffer,x
	dex
	bpl clearoutsid




	
resetmidival
	lda #$03
resetcontrol
	sta $de00


resetirq
	lda #$95

resetcontrol2
	sta $de00
resetexit
	rts
	

patch_sids
	lda haspatched
	bmi resetexit
	ldx #$00
patch_sids_loop
	stx drivernumber
	lda player_patchsid,x
	cmp #$ff
	bne patch_sids_continue
	lda player_readsidhi,x
	cmp #$ff
	bne patch_sids_dontskip
	jmp patch_sids_skip
patch_sids_dontskip
	lda player_patchreplacemem+$01
	sta player_readsidhi,x
	jmp patch_sids_skip
patch_sids_continue
	sta player_patchread+$02
	sta player_patchwrite+$02
	
	clc
	sbc #$00
	sta patch_compare+$02
	sta player_patchreadlo+$02
	sta player_patchwritelo+$02
	
	adc #$01
	clc
	adc player_patchsid_threshold,x
	sta player_patchcompare+$01
	
	lda patch_memory,x
	sta player_patchwritekey+$01
	sta player_patchreplacemem+$01
	lda patch_memorylo,x
	sta player_patchwritekeylo+01
	ldy #$00
player_patchread
	lda $1000,y
	cmp player_readsidhi,x
	bne player_patchskip
	
patch_compare
	lda $0ffe,y
	cmp #$8d
	beq player_patchwritekey
	cmp #$9d
	beq player_patchwritekey
	cmp #$99
	beq player_patchwritekey
	cmp #$8e
	beq player_patchwritekey
	cmp #$8c
	bne player_patchskip
player_patchwritekey
	lda #$e0
player_patchwrite
	sta $1000,y
	clc
player_patchreadlo
	lda $0fff,y
player_patchwritekeylo	
	adc #$00
player_patchwritelo
	sta $0fff,y
	iny
player_patchskip	
	iny
	cpy #$00
	bne player_patchread
	inc patch_compare+$02
	inc player_patchwrite+$02
	inc player_patchread+$02
	inc player_patchreadlo+$02
	inc player_patchwritelo+$02
	lda player_patchread+$02
	
player_patchcompare
	cmp #$00
	bne player_patchread	
player_patchreplacemem
	lda #$e0
	sta player_readsidhi,x

patch_sids_skip
	ldx drivernumber
	inx
	cpx player_patchmax 
	beq patch_sids_end
	jmp patch_sids_loop
patch_sids_end	
	
	lda #$ff
	sta haspatched
	rts

	org $c600

		
	; work out midi note from sid osc freq
	
findnoteval	

	lda tempnotehi

	cmp #$11
	bcc findlowerhalf
	cmp #$44
	bcs findupperhalf2
	
	ldx #$30
freqcheckhi
	cmp freqhi,x
	beq gothifreq
	bcc gothifreq1
	inx
	cpx #$48
	bne freqcheckhi
	ldx #$00
	sta midival
	rts
gothifreq1
	
gothifreq
	stx midival
	lda tempnotelo
freqchecklo	
	cmp freqlo,x
	beq gotallfreq
	bcc gotallfreq	
	inx
	jmp freqchecklo
	stx midival
gotallfreq
	cpx #$00
	beq gotskip
	txa
	clc
	adc #$0c
	sta midival	
gotskip	
	rts

zeronote
	sta midival
	rts

findupperhalf2
	ldx #$48
h2freqcheckhi
	cmp freqhi,x
	beq h2gothifreq
	bcc h2gothifreq1
	inx
	cpx #$60
	bne h2freqcheckhi
	ldx #$00
	sta midival
	rts
h2gothifreq1
	
h2gothifreq
	jmp gothifreq
	
findlowerhalf
	cmp #$04
	bcs findlowerhalf2
	
	ldx #$00
	
l2freqcheckhi
	cmp freqhi,x
	beq l2gothifreq
	bcc l2gothifreq1
	inx
	cpx #$17
	bne l2freqcheckhi
	ldx #$00
	sta midival
	rts
l2gothifreq1
	
l2gothifreq
	jmp gothifreq	
	
findlowerhalf2	
	ldx #$17
	
lfreqcheckhi
	cmp freqhi,x
	beq lgothifreq
	bcc lgothifreq1
	inx
	cpx #$30
	bne lfreqcheckhi
	ldx #$00
	sta midival
	rts
lgothifreq1
	
lgothifreq
	jmp gothifreq


hiloconv
	.byte $00,$00,$08,$10,$18,$20,$28,$30,$38


midibuffer = $c700
siddata = $c800
sidbuffer = $c900
	
	org $ca00
; sid track variables
midichancurrentchannel
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
midichancurrentpatch ; the sid patch that is being used
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
midichancurrentnote
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
midichancurrentvelocity
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	
; midi patch side variables
midirealpatch ; the actual patch on the midi device channel
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
midifixednotevalue
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
midifixedvelocity
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
midifixeddelay
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
midihasmodulation
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
midimodulationval
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	
miditranspose
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00


tempnotehi
	.byte $00
tempnotelo
	.byte $00
	

notedelay
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
moddelay
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
modplaying
	.byte $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
	
midioffsets
	.byte $00,$03,$06
chanoffsets
	.byte $00,$07,$0e,$20,$27,$2e,$40,$47,$4e

midichantemp
	.byte $00
	
midilen
	.byte $00
	
gateset
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	

patch_memory
	.byte $c8,$c8,$c8,$c8
player_readsidhi
	.byte $d4,$d4,$d4,$d4

player_patchmax
	.byte $00

	org $cb20

writesid

	ldx buffercurrent
	ldy bufferoffset,x
	ldx #$00
copytobuffer
	lda $c800,x
	sta sidbuffer,y
	inx
	iny
	cpx #$19
	bne copytobuffer
	
	inc buffercurrent
	lda buffercurrent
	and #$07
	sta buffercurrent
	
	inc buffernumber
	lda buffernumber
	and #$07
	sta buffernumber


	ldx buffernumber
	lda bufferoffset,x
	tax
	
	lda sidbuffer,x
	sta $d400
	lda sidbuffer+$01,x
	sta $d401
	lda sidbuffer+$02,x
	sta $d402
	lda sidbuffer+$03,x
	sta $d403
	lda sidbuffer+$04,x
	sta $d404
	lda sidbuffer+$05,x
	sta $d405
	lda sidbuffer+$06,x
	sta $d406

	lda sidbuffer+$07,x
	sta $d407
	lda sidbuffer+$08,x
	sta $d408
	lda sidbuffer+$09,x
	sta $d409
	lda sidbuffer+$0a,x
	sta $d40a
	lda sidbuffer+$0b,x
	sta $d40b
	lda sidbuffer+$0c,x
	sta $d40c
	lda sidbuffer+$0d,x
	sta $d40d
	
	lda sidbuffer+$0e,x
	sta $d40e
	lda sidbuffer+$0f,x
	sta $d40f
	lda sidbuffer+$10,x
	sta $d410
	lda sidbuffer+$11,x
	sta $d411
	lda sidbuffer+$12,x
	sta $d412
	lda sidbuffer+$13,x
	sta $d413
	lda sidbuffer+$14,x
	sta $d414
	 
	lda sidbuffer+$15,x
	sta $d415
	lda sidbuffer+$16,x
	sta $d416
	lda sidbuffer+$17,x
	sta $d417
	lda sidbuffer+$18,x
	sta $d418
	rts
	
	org $cc00

	; Frequency table (by Mr.Sid from Codebase64)
	
		.IF NTSC = 0
freqlo
	        ;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e  ; 1
                .byte $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c  ; 2
                .byte $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37  ; 3
                .byte $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e  ; 4
                .byte $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc  ; 5
                .byte $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8  ; 6
                .byte $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71  ; 7
                .byte $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,$ff  ; 8

freqhi
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02  ; 1
                .byte $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04  ; 2
                .byte $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08  ; 3
                .byte $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10  ; 4
                .byte $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20  ; 5
                .byte $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41  ; 6
                .byte $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83  ; 7
                .byte $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,$ff  ; 8
		.ENDIF
		
		.IF NTSC = 1
freqlo
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $0c,$1c,$2d,$3f,$52,$66,$7b,$92,$aa,$c3,$de,$fa  ; 1
                .byte $18,$38,$5a,$7e,$a4,$cc,$f7,$24,$54,$86,$bc,$f5  ; 2
                .byte $31,$71,$b5,$fc,$48,$98,$ee,$48,$a9,$0d,$79,$ea  ; 3
                .byte $62,$e2,$6a,$f8,$90,$30,$dc,$90,$52,$1a,$f2,$d4  ; 4
                .byte $c4,$c4,$d4,$f0,$20,$60,$b8,$20,$a4,$34,$e4,$a8  ; 5
                .byte $88,$88,$a8,$e0,$40,$c0,$70,$40,$48,$68,$c8,$50  ; 6
                .byte $10,$10,$50,$c0,$80,$80,$e0,$80,$90,$d0,$90,$a0  ; 7
                .byte $20,$20,$a0,$80,$00,$00,$c0,$00,$20,$a0,$20,$40  ; 8
     
freqhi
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01  ; 1
                .byte $02,$02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03  ; 2
                .byte $04,$04,$04,$04,$05,$05,$05,$06,$06,$07,$07,$07  ; 3
                .byte $08,$08,$09,$09,$0a,$0b,$0b,$0c,$0d,$0e,$0e,$0f  ; 4
                .byte $10,$11,$12,$13,$15,$16,$17,$19,$1a,$1c,$1d,$1f  ; 5
                .byte $21,$23,$25,$27,$2a,$2c,$2f,$32,$35,$38,$3b,$3f  ; 6
                .byte $43,$47,$4b,$4f,$54,$59,$5e,$64,$6a,$70,$77,$7e  ; 7
                .byte $86,$8e,$96,$9f,$a9,$b3,$bd,$c9,$d5,$e1,$ef,$fd  ; 8
                
                .ENDIF
                
	org $cd00

drivernumber
	.byte $00

buffernumber
	.byte $00
buffercurrent
	.byte $00
bufferoffset
	.byte $00,$20,$40,$60,$80,$a0,$c0,$e0

midicartdata
	; sequential
	.byte $00,$95,$02,$01,$03,$00,$00,$00
	; datel
	.byte $04,$96,$06,$05,$03,$00,$00,$00

patchoffsetslo
	.byte #<patchdatatrack1,#<patchdatatrack2,#<patchdatatrack3,#<patchdatatrack4,#<patchdatatrack5,#<patchdatatrack6,#<patchdatatrack7,#<patchdatatrack8,#<patchdatatrack9
patchoffsetshi
	.byte #>patchdatatrack1,#>patchdatatrack2,#>patchdatatrack3,#>patchdatatrack4,#>patchdatatrack5,#>patchdatatrack6,#>patchdatatrack7,#>patchdatatrack8,#>patchdatatrack9
		
volumetable
	.byte 0,10,18,32,40,48,50,58,60,68,70,80,90,100,107,117,127                

sustaintable
	.byte $01,$02,$04,$06,$09,$0c,$0f,$11,$15,$18,$1f,$20,$24,$2c,$34,$3f

haspatched
	.byte $00
	
tempplayer_startdelay
	.byte $00,$00,$00,$00
	
skipinst
	.byte $00
	
temp
	.byte $00
	
midival
	.byte $00

midisendcount 
	.byte $00

	.include mididata.asm
	